// main for mkevals version 2.3
// (revision date: 29 june 2005)
//
#include "mkevals.h"

int main(int argc, char** argv)
{

    extern int optind;
    misc m;
    misc* mp = &m;
    string line;
    string header;
    uns cnt = 0;
    map<string, uns> ngr;                  // counts grades
    UnsStrMMap mm;                         // multimap for ranking class

    m.upgrade = no;                        // default: don't discount 
    m.upgradeflag = off;                   // early low midterm scores
    m.plus = no;                           // default: no + or - grades
    m.rank = no;                           // default: no rank page
    m.debug = off;                         // default: no extra output
    m.old = no;                            // default: use new class rosters

    get_opts(argc, argv, mp);
    if ((argc - optind) != 1) {
        prn_usage(argv);
        exit(1);
    }

    matrix bp(0, vector<dbl>(nbpts));      // breakpoint array
    vector<dbl> wt;                        // item weighting factors
    get_template_data(mp, bp, wt, argv);   // gets bps and wts etc
    vector<dbl> total(nbpts);              // total: vector of zeros
    bp.push_back(total);                   // add a row to bp for totals
    normalize_wts(mp, wt);                 // normalize weight vector
    vector<uns> cat(bp.size());            // grade category vector
    vector<dbl> item(bp.size());           // item scores for a student
    calc_tot_bps(bp, wt, m.items);         // calculate total bps
    string ifn = argv[optind];             // input file name
    string ofn = ifn + ".evs";             // output file name
    string ofngr = ifn + ".grd";           // grade page file
    string ofnrank = ifn + ".rank";        // rank page file
    remove(ofn.c_str());                   // remove any old files
    remove(ofngr.c_str());                 // lying around
    remove(ofnrank.c_str());
    ifstream ifs(ifn.c_str());             // open input file
    gassert(ifs);                          // ensure input file exists
    ofstream ofs(ofn.c_str());             // open output file
    ofs << nroff_cmds;                     // nroff commands to ofn
    ofstream ofsgr(ofngr.c_str());         // open output grade page file

    for (uns n = 1 ; getline(ifs, line) ; n++){// start of main loop
        if (line.find("#") < string::npos)     // comment line
            continue;                          // so ignore it
        if (line.find("@@@@") < string::npos)  // course line
            ofs << line << "\n";               // so print it in ofn
        if (line.find("&&&&") < string::npos){ // header line
            header = line;                     // so save it
            string name = getname(header);     // get name from header
            setpronouns(header);               // set pronouns
            cnt++;                             // count header lines
            for (uns j = 0 ; j < m.items ; ++j)// get item scores
                ifs >> item[j];
            if (!ifs)
                err("missing score for ", name, " after line ", n);
            get_total(mp, item, wt);
            findcats(bp, item, cat, 0, m.items);// categorize exam scores
            if (m.debug) show_details(mp, name, bp, item, wt, cat);
            if (m.upgrade){              // discount low mids or quizzes
                upgrade(mp, bp, item, wt, cat);
            }
            ngr = write_eval(mp, ofs, item, header, cat);
            togradepage(mp, header, ofsgr, cat.back());
            if (m.rank){                 // rank students if desired
                rank(mp, mm, header, cat.back());
            }
        }
    }                                          // end of main loop
    if (m.rank){
        mk_rank_file(mm, ofnrank);
    }


    cout << "\n\nGood.  It worked.\n\n"
        "You have drafted " << cnt << " evaluations.\n"
        "Here is the distribution for the\n"
        "suggested assigned grades:\n\n";

    if (m.plus) showstat(ngr, "A+");
    showstat(ngr, "A");
    if (m.plus) showstat(ngr, "A-");
    if (m.plus) showstat(ngr, "B+");
    showstat(ngr, "B");
    if (m.plus) showstat(ngr, "B-");
    if (m.plus) showstat(ngr, "C+");
    showstat(ngr, "C");
    showstat(ngr, "D");
    showstat(ngr, "F");
    showstat(ngr, "P");
    showstat(ngr, "NP");
    showstat(ngr, "W");
    cout << "\n";

    ifs.close();  // close input file ifn
    ofs.close();  // close ".evs" file

    mk_roff_file(ofn);  // now nroff the ".evs" file and then
                        // remove ".evs" and other unneeded
                        // files

    cout << "\nA list of suggested grades will be found in\n"
        "the file \"" << ofngr << "\".\n\n"
        "The completed evaluations will be found in\n"
        "the file \"" << ifn + ".roff" << "\", ready for any editing \n"
        "before sending to the registrar at script@ucsc.edu.\n";
    if (m.rank){
        cout << "\nThe file \"" << ofnrank << "\" contains a class\n"
            "list showing the ranking by category.\n\n";
    }
    else cout << "\n";
}

